#!/bin/sh
#
# Purpose:
#   Retrieves studies from archive.
#
# Synopsis:
#   get.sh [-f] study
#
# Arguments:
#     -f    :  force retrieving obsoleted studies/files
#     study :  Study Instance UID of the study to be retrieved
#
# Description:
#  OBSOLETE [7.0]
#
#   Retrieves study from the archive. Will not retrieve obsoleted
# studies/files unless the -f option is specified.
#
# ENDDOC
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

. $HOME/etc/setenv.sh

# if get.sh is called directly:
# run as a task instead
if [ `basename $0` = "get.sh" ] ; then
	$TASKD_HOME/runshellcmd.sh -R -r 0 $0 "$@"
	exit $?
fi

LOGGER="/usr/bin/logger"
TAG="archive/get[$$]"

# force retrieving obsoleted studies/files
FORCE=""

if [ "$1" = "-f" ] ; then
	FORCE="-f"
	shift
fi

study=$1

# include nettask.shi for checking if the device is temporary down
. $TASKD_HOME/nettask.shi

# override getPeer_() of nettask.shi
getPeer_()
{
	echo "@Archive @Archive-GET"
}

run_()
{
	if [ -z "$study" ] ; then
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "Missing argument"
		return 125
	fi

	$TASKD_HOME/runjavacmd -c "0 archive.Get $FORCE -s \"$study\"" -p "$JAVA_SERVER_PORT"
	ERR=$?

	exit $ERR
}


# override print_
print_()
{
	$TASKD_HOME/runjavacmd -c "print archive.Get $FORCE -s \"$study\"" -p "$JAVA_SERVER_PORT"
}

#
# Print value to be displayed on the WEB GUI
#
get_display_name_()
{
	echo "Retrieve study from EP archive" 
}

#
# Print resources used by the task 
#
get_resources_()
{
	echo "System.NET.@Archive.GET"
}

#
# Print property value
#
get_property_()
{
	if [ "$PROPERTY" = "1PST" ] ; then
		echo -n "$study"
		return 0
	fi
	 
	if [ "$PROPERTY" = "1PPE" ] ; then
		echo -n "@Archive" 
		return 0
	fi

	get_def_property_
}

