#!/bin/sh
#
# Purpose:
#   Retrieves an objects from archive.
#
# Synopsis:
#   mergeobject.sh study objectFile studyArchDir
#
# Arguments:
#     study :  Study Instance UID of the study to be retrieved
#     objectFile : The filename of the object file to be retrieved
#     studyArchDir : the absolute directory in the archive where the study is
#
# Description:
#
#   Retrieves an object from the archive. 
#
# ENDDOC
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

. $HOME/etc/setenv.sh

# if mergeobject.sh is called directly:
# run as a task instead
if [ `basename $0` = "mergeobject.sh" ] ; then
	$TASKD_HOME/runshellcmd.sh -R -r 0 $0 "$@"
	exit $?
fi

LOGGER="/usr/bin/logger"
TAG="archive/mergeobject[$$]"

study=$1
objectFile=$2
studyArchDir=$3

# include nettask.shi for checking if the device is temporary down
. $TASKD_HOME/nettask.shi

# override getPeer_() of nettask.shi
getPeer_()
{
	echo "@Archive @Archive-GET"
}

run_()
{
	
	if [ -z "$objectFile" -o -z "$study" -o -z "$studyArchDir" ] ; then
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "missing parameter(s)"
		return 125
	fi

	$TASKD_HOME/runjavacmd -c "0 archive.MergeObject -s \"$study\" -f \"$objectFile\" -d \"$studyArchDir\"" -p "$JAVA_SERVER_PORT"
	ERR=$?

	exit $ERR
}


# override print_
print_()
{
	$TASKD_HOME/runjavacmd -c "print archive.MergeObject -s \"$study\" -f \"$objectFile\" -d \"$studyArchDir\"" -p "$JAVA_SERVER_PORT"
}

#
# Print value to be displayed on the WEB GUI
#
get_display_name_()
{
	echo "Merge object from legacy EP archive to enterprise archive" 
}

#
# Print resources used by the task 
#
get_resources_()
{
	echo "System.NET.@Archive.GET"
}

#
# Print property value
#
get_property_()
{
	if [ "$PROPERTY" = "1PST" ] ; then
		echo -n "$study"
		return 0
	fi
	 
	if [ "$PROPERTY" = "1PPE" ] ; then
		echo -n "@Archive" 
		return 0
	fi

	get_def_property_
}
