#!/bin/sh
#
# Component: cases
#
# Description:
#    Checks if there is enough disk space, and makes space if necessary
#
# Usage:
#   checkspace.sh [clean]
#
# If 'clean' is specified as the first argument, checkspace will call makespace.jsp
# to make enough space by removing oldest studies.
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

. $HOME/etc/setenv.sh

LOGGER="/usr/bin/logger"
TAG="checkspace[$$]"


#
# Check if another instance is running
#
# Should be re-visited after OS upgrades
#

if [ "$1" = "-n" ] ; then
	# no check - started from another instance via MakeSpaceTask 
	shift
	$LOGGER -p $LOG_FACILITY.info -t "$TAG" "Don't check for other instances"
else
	mypath=$(readlink -f $0)
	mypid=$$
	myname=$(basename $mypath)
	
	# Sending ps list to file to avoid subprocesses be created by pipes and listed in the ps list
	ps -C "$myname" hwww -o pid,cmd= > $TMP/testcheckspace.$mypid
	cat $TMP/testcheckspace.$mypid | while read procPid procShell procCmd procArgs; do
		# procArgs is only needed to ignore additional strings on ps list
		if [ "$procPid" = "$mypid" ] ; then
			# myself - skip"
			continue
		fi
	
		if [ "$procShell" != "/bin/sh" ] ; then
			# Not a script
			continue
		fi
	
		if [ "${procCmd#/}" = "${procCmd}" ] ; then
			# relative path
			procCwd=`readlink -f /proc/$procPid/cwd`
			if [ $? -ne 0 ] ; then
				# either the process was started from a directory the current process don't have access for
				# or the process have just gone
	 			$LOGGER -p $LOG_FACILITY.err -t "$TAG" "Process $procPid could not be checked: $( ps h -p $procPid u)"
				exit 1
			fi
			cmdLine=`readlink -f "$procCwd/${procCmd}"`
		else
			# absolute path
			cmdLine=`readlink -f "${procCmd}"`
		fi
	
		if [ "$mypath" = "$cmdLine" ] ; then
	 		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "Another instance is running: $( ps h -p $procPid u)"
			exit 1
		else
	 		$LOGGER -p $LOG_FACILITY.info -t "$TAG" "Different process $cmdLine: $( ps h -p $procPid u)"
		fi
	done
	
	chkRes=$?
	
	rm $TMP/testcheckspace.$mypid
	if [ $chkRes -ne 0 ] ; then
		exit 1
	fi
fi

#
# Do the real job
#

$LOGGER -p $LOG_FACILITY.info -t "$TAG" "Starts disk space calculation [$1]"

touch $VAR/diskCheckSpaceInProgress

if [ "$1" = "clean" ] ; then
	$TASKD_HOME/runjavacmd -c "0 cases.DailyCleanerTask |Purge|" -p "$JAVA_SERVER_PORT"
fi

# -L to count symbolic links size

DISK_INFO_TMP=$TMP/disk_info.$$

echo "CACHE:" >> $DISK_INFO_TMP
$REPOSITORYHANDLER_HOME/scripts/getFSStats.sh $REPOSITORY_CACHE >> $DISK_INFO_TMP
if [ $? -ne 0 ] ; then
	rm $VAR/diskCheckSpaceInProgress
	exit 1
fi
echo "PROCESSED:" >> $DISK_INFO_TMP
$REPOSITORYHANDLER_HOME/scripts/getFSStats.sh $REPOSITORY_PROCESSED >> $DISK_INFO_TMP
if [ $? -ne 0 ] ; then
	rm $VAR/diskCheckSpaceInProgress
	exit 1
fi
echo "DATA:" >> $DISK_INFO_TMP
$REPOSITORYHANDLER_HOME/scripts/getFSStats.sh $REPOSITORY_DICOM_DATA >> $DISK_INFO_TMP
if [ $? -ne 0 ] ; then
	rm $VAR/diskCheckSpaceInProgress
	exit 1
fi
echo "# -- Complete --" >> $DISK_INFO_TMP
mv $DISK_INFO_TMP $VAR/disk_info

rm $VAR/diskCheckSpaceInProgress

if [ "$1" = "clean" ] ; then
	$LOGGER -p $LOG_FACILITY.info -t "$TAG" "Starts MakeSpaceTask"
	$TASKD_HOME/runjavacmd -c "0 imagemed.util.MakeSpaceTask -n" -p "$JAVA_SERVER_PORT"

	# removed CBT 2664	$REPOSITORYHANDLER_HOME/bin/checkOverload $REPOSITORY_DICOM_DATA

	# removed CBT 2664  $REPOSITORYHANDLER_HOME/bin/checkOverload $REPOSITORY_PROCESSED
fi

$LOGGER -p $LOG_FACILITY.info -t "$TAG" "Ends"
