var radarPanel = null;

function openRadar( width, height ) {
	if ( radarPanel == null ) {
		// radarPanelURL comes from global variable set in reportview.jsp

		radarPanel = new DraggableHoverDiv( true, true );
		radarPanel.iframe = $( '<iframe/>' );
		radarPanel.iframe.attr( 'src', radarPanelURL );
		radarPanel.iframe.width( width );
		radarPanel.iframe.height( height );
		radarPanel.iframe.appendTo( radarPanel.contentDiv );
	}
	else {
		try {
			radarPanel.iframe.get()[ 0 ].contentDocument.location.reload( true );
		}
		catch ( e ) {
			radarPanel.iframe.attr( 'src', radarPanel.iframe.attr( 'src' ) );
		}
	}
	radarPanel.position();
	radarPanel.show();
};

function closeRadar() {
	radarPanel.hide();
};

var cannedReportNames = new Array();
var cannedReportMap = new Array();
var CRFIELDS = [ "int0_text", "impression", "int0_note" ];
var UTF8PREFIX = "encode_mode=utf8&";

function CannedReport( name, content, props ) {
	cannedReportNames.push( name );
	cannedReportMap[ name ] = this;
	this.name = name;
	this.content = content;
	this.props = props ? props : {};

	this.apply = function() {
		var utf8 = this.content && this.content.startsWith( UTF8PREFIX );
		setInputValues( utf8 ? this.content.substr( UTF8PREFIX.length )
				: this.content, utf8 );
	}

	this.initContent = function() {
		var parts = new Array();
		pushFields( parts, CRFIELDS, true );
		this.content = UTF8PREFIX + parts.join( "&" );
	}

	this.setProp = function( name, val ) {
		this.props[ name ] = val;
		this.isActive = this.isActive_();
	}

	this.isActive_ = function() {
		if ( this.props[ "modalities" ] ) {
			var modArray = this.props[ "modalities" ].split( "\n" );
			for ( var i = 0; i < modArray.length; i++ ) {
				var allowedMod = modArray[ i ];
				for ( var j = 0; j < currentModalities.length; j++ ) {
					if ( allowedMod == currentModalities[ j ] ) {
						return true;
					}
				}
			}
			return false;
		}
		return true;
	}

	this.pushToServer = function( overwrite ) {
		var postData = {
			cFromBrowser : "true",
			cOper : ( overwrite ? "overwrite" : "write" ),
			cTemplate : this.name,
			cContent : this.content
		};
		if ( this.props[ "modalities" ] ) {
			postData.cModality = this.props[ "modalities" ];
		}
		$.ajax( {
			type : "POST",
			url : "/cases/reportTemplateManager.jsp",
			data : postData,
			crTemplateName : this.name,
			cCreated : overwrite ? false : true,
			success : function() {
				crPanel.saved( this.crTemplateName, this.cCreated );
			},
			error : function() {
				crPanel.removed( this.crTemplateName ); // remove template from cannedReportNames and cannedReportMap
				alert( LangResMap.CANNEDREPORT_SAVE_FAILED );
			}
		} );
	}

	this.deleteFromServer = function( overwrite ) {
		var postData = {
			cFromBrowser : "true",
			cOper : "delete",
			cTemplate : this.name
		};

		$.ajax( {
			type : "POST",
			url : "/cases/reportTemplateManager.jsp",
			data : postData,
			crTemplateName : this.name,
			success : function() {
				crPanel.removed( this.crTemplateName );
				//alert( "success" );
			},
			error : function() {
				alert( LangResMap.CANNEDREPORT_DELETE_FAILED );
			}
		} );
	};

	this.isActive = this.isActive_();
}

var crPanel = false;

var stickAbbreviation = false;

function CannedReportPanel() {
	if ( crPanel ) {
		return;
	}

	this.loadedTemplate = null;

	this.setLoadedTemplate = function( lt ) {
		this.loadedTemplate = lt;
		$( "#crSelect_flexselect" ).val( lt );
		this.checkButtons();
	}

	this.rootDiv = $( '#crDiv' );
	this.rootDiv.focusout( function( event ) {
		if ( isIE ) {
			// check for false focusout (fired for dragging the scrollbar)
			var target = event.target;
			var inputField = $( "#crSelect_flexselect" )[ 0 ];
			if ( target === inputField ) {
				return;
			}
		}
		setTimeout( function() {
			if ( !stickAbbreviation ) {
				$( "#crSelect_flexselect" ).val( crPanel.loadedTemplate );
				crPanel.checkButtons();
			}
			$( "#crSelect_flexselect_dropdown" ).hide(); // sometimes it seems to stick...
		}, 500 )
	} );

	this.rootDiv.hide();
	var table = $( '<table cellspacing=0 cellpadding=0>' );
	table.appendTo( this.rootDiv );
	var row = $( '<tr>' );
	row.appendTo( table );
	var td1 = $( '<td>' );
	td1.appendTo( row );
	var td2 = $( '<td>' );
	td2.appendTo( row );
	var td3 = $( '<td>' );
	td3.appendTo( row );
	var td4 = $( '<td>' );
	td4.appendTo( row );
	this.select = document.createElement( 'select' );
	td1.append( this.select );
	$( this.select ).attr( "id", "crSelect" );
	for ( var i = 0; i < cannedReportNames.length; i++ ) {
		if ( !cannedReportMap[ cannedReportNames[ i ] ].isActive ) {
			continue;
		}
		var option = document.createElement( "option" );
		option.value = cannedReportNames[ i ];
		option.text = cannedReportNames[ i ];
		this.select.appendChild( option );
	}
	$( this.select ).addClass( "flexselect" );
	//	$( this.select ).change( function() {
	//		// TODO: confirm loading...
	//		cannedReportMap[ $( crPanel.select ).val() ].apply();
	//	} );
	addLoadEvent( function() {
		jQuery( "select.flexselect" ).flexselect();
		$( '#crDiv' ).show();
	} );

	this.loadButton = $( "<div>" );
	this.loadButton.appendTo( td2 );
	this.loadButton.addClass( "CssButton Svg APPROVE_DEFAULT" );
	this.loadButton.attr( "title", LangResMap.CANNEDREPORT_LOAD );
	this.loadButton.mousedown( function() {
		crPanel.load();
	} );
	this.loadButton.hide();
	this.loadButtonD = $( "<div>" );
	this.loadButtonD.appendTo( td2 );
	this.loadButtonD.addClass( "CssButton Svg APPROVE_DEFAULT disabled" );
	this.loadButtonD.attr( "title", LangResMap.CANNEDREPORT_LOAD );

	this.loadDialog = new LoadDialog( this.rootDiv );
	this.loadedTemplate = "";

	this.load = function( confirmed ) {
		if ( !confirmed ) {
			// check for non-empty fields
			for ( var i = 0; i < CRFIELDS.length; i++ ) {
				var val = getInputFieldContent( CRFIELDS[ i ] );
				if ( val && val.trim() != '' ) {
					this.loadDialog.show();
					return;
				}
			}
		}
		this.setLoadedTemplate( $( "#crSelect_flexselect" ).val() );
		cannedReportMap[ this.loadedTemplate ].apply();
	}

	this.saveButton = $( "<div>" );
	this.saveButton.appendTo( td3 );
	this.saveButton.addClass( "CssButton Svg SAVE" );
	this.saveButton.attr( "title", LangResMap.CANNEDREPORT_SAVE );
	this.saveButton.mousedown( function() {
		crPanel.saveDialog.init();
		crPanel.saveDialog.show();
	} );
	this.saveButton.hide();
	this.saveButtonD = $( "<div>" );
	this.saveButtonD.appendTo( td3 );
	this.saveButtonD.addClass( "CssButton Svg SAVE disabled" );
	this.saveButtonD.attr( "title", LangResMap.CANNEDREPORT_SAVE );

	this.saveDialog = new SaveDialog( this.rootDiv );

	this.save = function( newName ) {
		var overwrite = cannedReportNames.indexOf( newName ) != -1;
		// save new
		var newCR = overwrite ? cannedReportMap[ newName ] : new CannedReport(
				newName );
		newCR.initContent(); // load content
		this.saveDialog.applyTo( newCR ); // get properties
		// submit to server...
		newCR.pushToServer( overwrite );
	}

	this.saved = function( template, created ) {
		if ( created ) {
			var option = document.createElement( "option" );
			option.value = template;
			option.text = template;
			this.select.appendChild( option );
			$( this.select ).val( template );
			$( this.select ).flexselect( {
				preSelection : true
			} );
		}
		this.setLoadedTemplate( template );
	}

	this.deleteButton = $( "<div>" );
	this.deleteButton.appendTo( td4 );
	this.deleteButton.addClass( "CssButton Svg DELETE" );
	this.deleteButton.attr( "title", LangResMap.CANNEDREPORT_DELETE );
	this.deleteButton.mousedown( function() {
		crPanel.deleteDialog.init();
		crPanel.deleteDialog.show();
	} );
	this.deleteButton.hide();
	this.deleteButtonD = $( "<div>" );
	this.deleteButtonD.appendTo( td4 );
	this.deleteButtonD.addClass( "CssButton Svg DELETE disabled" );
	this.deleteButtonD.attr( "title", LangResMap.CANNEDREPORT_DELETE );

	this.deleteDialog = new DeleteDialog( this.rootDiv );

	this.deleteCR = function() {
		// delete now
		var delName = $( "#crSelect_flexselect" ).val();
		cannedReportMap[ delName ].deleteFromServer();
		if ( this.loadedTemplate == delName ) {
			this.setLoadedTemplate( "" );
		}
	};
	this.removed = function( delName ) {
		$( this.select ).children( 'option[value=\'' + delName + '\']' )
				.remove();
		$( this.select ).flexselect( {
			preSelection : false
		} );
		$( "#crSelect_flexselect" ).val( "" );
		var index = cannedReportNames.indexOf( delName );
		if ( index > -1 ) {
			cannedReportNames.splice( index, 1 );
		}
		delete cannedReportMap[ delName ];
		this.checkButtons();
	};

	this.checkButtons = function() {
		var val = $( "#crSelect_flexselect" ).val();
		if ( cannedReportNames.indexOf( val ) != -1
				&& cannedReportMap[ val ].isActive ) {
			//existing report
			this.deleteButtonD.hide();
			this.deleteButton.show();
			this.loadButtonD.hide();
			this.loadButton.show();
		}
		else {
			this.deleteButton.hide();
			this.deleteButtonD.show();
			this.loadButton.hide();
			this.loadButtonD.show();
		}
		// check if val is a valid teplate name
		if ( val.length > 0 && val == removeInvalidChars( val ) ) {
			this.saveButtonD.hide();
			this.saveButton.show();
		}
		else {
			this.saveButton.hide();
			this.saveButtonD.show();
		}
	};

	this.rootDiv.addClass( "cannedReportPanel" );
	crPanel = this;
}

var DialogTemplate = function( parentDiv, dialogName ) {
	this.rootDiv = $( "<div class=cannedReportConfirm>" );
	this.rootDiv.css( {
		'position' : 'absolute',
		'zIndex' : 11
	} );
	this.rootDiv.hide();
	this.rootDiv.appendTo( parentDiv );
	this.title = $( "<div>" );
	this.title.appendTo( this.rootDiv );

	this.setTitle = function( txt ) {
		this.title.html( "<b>&nbsp;" + txt + "&nbsp;</b>" );
	}

	this.contentDiv = $( "<div>" );
	this.contentDiv.appendTo( this.rootDiv );

	$(
			"<table cellspacing=0 cellpadding=0><tr><td width=100%>&nbsp;</td><td><div id='cr"
					+ dialogName
					+ "DialogDoBtn'></td>"
					+ "<td><div id='cr"
					+ dialogName
					+ "DialogCancelBtn'></td></tr></table>" ).appendTo( this.rootDiv );
	this.doButton = $( "#cr" + dialogName + "DialogDoBtn" );
	this.doButton.addClass( "CssButton Svg APPROVE_DEFAULT" );

	this.cancelBtn = $( "#cr" + dialogName + "DialogCancelBtn" );
	this.cancelBtn.addClass( "CssButton Svg CLOSE" );

	this.show = function() {
		stickAbbreviation = true;
		document.body.appendChild( getMshShader() );
		this.rootDiv.show();
	};

	this.hide = function() {
		document.body.removeChild( getMshShader() );
		this.rootDiv.hide();
	};

}

function SaveDialog( parentDiv ) {
	DialogTemplate.call( this, parentDiv, 'Save' );
	$(
			"<div><input type=checkbox id='crModSelCB'>"
					+ LangResMap.CANNEDREPORT_SELECT_MODALITIES + "</div>" )
			.appendTo( this.contentDiv );
	this.checkbox = $( "#crModSelCB" );
	this.checkbox.change( function() {
		crPanel.saveDialog.select
				.prop( 'disabled', this.checked ? false : true );
	} );
	this.select = $( "<select multiple>" );
	this.select.css( 'float', 'left' );
	this.select.css( 'width', '100%' );
	for ( var i = 0; i < currentModalities.length; i++ ) {
		this.select.append( $( '<option>', {
			value : currentModalities[ i ],
			text : currentModalities[ i ]
		} ) );
	}
	this.select.appendTo( this.contentDiv );
	this.doButton.click( function() {
		stickAbbreviation = false;
		crPanel.save( crPanel.saveDialog.newName );
		crPanel.saveDialog.hide();
	} );

	this.cancelBtn.click( function() {
		stickAbbreviation = false;
		crPanel.saveDialog.hide();
		crPanel.rootDiv.focusout();
	} );

	this.init = function() {
		this.newName = $( "#crSelect_flexselect" ).val();
		var cr = cannedReportMap[ this.newName ];
		this.setTitle( cr ? LangResMap.CANNEDREPORT_OVERWRITE_CONFIRM
				: LangResMap.CANNEDREPORT_SAVE_CONFIRM );
		if ( cr && cr.isActive && cr.props[ "modalities" ] ) {
			this.checkbox.prop( 'checked', true );
			this.select.prop( 'disabled', false );
			this.select.val( cr.props.modalities.split( "\n" ) );
		}
		else {
			this.checkbox.prop( 'checked', false );
			this.select.find( 'option:selected' ).prop( "selected", false );
			this.select.prop( 'disabled', true );
		}

	};

	this.applyTo = function( cr ) {
		if ( this.checkbox.prop( 'checked' ) && this.select.val() ) {
			cr.props.modalities = this.select.val().join( '\n' );
		}
		else {
			delete cr.props.modalities;
		}
	}
}

function LoadDialog( parentDiv ) {
	DialogTemplate.call( this, parentDiv, 'Load' );
	this.setTitle( LangResMap.CANNEDREPORT_LOAD_CONFIRM );
	this.doButton.click( function() {
		stickAbbreviation = false;
		crPanel.load( true );
		crPanel.loadDialog.hide();
	} );

	this.cancelBtn.click( function() {
		stickAbbreviation = false;
		crPanel.loadDialog.hide();
		crPanel.rootDiv.focusout();
	} );
}

function DeleteDialog( parentDiv ) {
	DialogTemplate.call( this, parentDiv, 'Delete' );
	this.doButton.click( function() {
		stickAbbreviation = false;
		crPanel.deleteCR();
		crPanel.deleteDialog.hide();
	} );
	this.cancelBtn.click( function() {
		stickAbbreviation = false;
		crPanel.deleteDialog.hide();
		crPanel.rootDiv.focusout();
	} );

	this.init = function() {
		var delName = $( "#crSelect_flexselect" ).val();
		this.setTitle( LangResMap.CANNEDREPORT_DELETE_CONFIRM.replace( '{0}',
				delName ) );
	}
}

function handleAbbreviationChange() {
	crPanel.checkButtons();
}

/*
CNameCheck::ECH CNameCheck::RemoveInvalidCharsECH(CString& strReportTemplateID)
{
//  warning C4996: Consider using strcpy_s instead...
#pragma warning (disable:4996)
	ECH bRet = UNCHANGED;
	char* pReportTemplateID = strReportTemplateID.GetBuffer(0);
	char* p = pReportTemplateID;
	while ((*p != '\0') && !::IsSpecAlNum(*p))
	{
		bRet = CHANGED;
		strcpy(p, p+1);
	}
	while (*p != '\0')
	{
		if ( IsValidNameChar( *p ) )
		{
			p++;
		}
		else
		{
			strcpy(p, p+1);
			bRet = CHANGED;
		}
	}
	strReportTemplateID.ReleaseBuffer();
	return bRet;
#pragma warning( pop )
}

static BOOL IsSpecAlNum(const char ch)
{
	return (((ch >= '0') && (ch <= '9')) ||
			((ch >= 'A') && (ch <= 'Z')) ||
			((ch >= 'a') && (ch <= 'z')) ||
			(((BYTE)ch >= 0x80) && ((BYTE)ch <= 0xff)) ||
			(ch == '_'));
}

// if this ever changes to allow '*' then check usage of '*'
// e.g. in STemplateProp::getDefaultName()
bool CNameCheck::IsValidNameChar( const char ch )
{
	return (::IsSpecAlNum(ch) || (ch == '-') || (ch == '#') || (ch == '.') || (ch == ' '));
}

	static char cForbidden[] = { '/', '\\', '?', '%', '*', ':', '|', '"', '<', '>',
								 '&', '#', '\'', ';', '=', '!' };
 */

function removeInvalidChars( name ) {
	var res = "";
	var i = 0;
	while ( i < name.length && !checkNameChar( name.charCodeAt( i ), true ) ) {
		i++;
	}
	while ( i < name.length ) {
		if ( checkNameChar( name.charCodeAt( i ), false ) ) {
			res += name.charAt( i );
		}
		i++;
	}
	return res;
}

function checkNameChar( cc, isFirst ) {
	var ch = String.fromCharCode( cc );
	var goodFirst = ch >= '0' && ch <= '9' || ch >= 'A' && ch <= 'Z'
			|| ch >= 'a' && ch <= 'z' || cc >= 0x80 && cc <= 0xff || ch == '_';
	if ( goodFirst || isFirst ) {
		return goodFirst;
	}
	// for the non-first character we allow some other characters
	return ch == '-' || ch == '#' || ch == '.' || ch == ' ';
}
/**/

/**
 * 
 * Dictation player handler
 * 
 */

var dictationPanels = new Array();

function KeyDescriptor( keyCode, isShift, isCtrl, isAlt, isMeta ) {
	this.keyCode = keyCode;
	this.isShift = !!( isShift ); // double negative is for converting undefined to false
	this.isCtrl = !!( isCtrl );
	this.isAlt = !!( isAlt );
	this.isMeta = !!( isMeta );
	this.matches = function( e ) {
		if ( this.keyCode && e && e.keyCode == this.keyCode ) {
			return ( this.isShift == !!e.shiftKey )
					&& ( this.isCtrl == !!e.ctrlKey )
					&& ( this.isAlt == !!e.altKey )
					&& ( this.isMeta == !!e.metaKey );
		}
		return false;
	};
}

var KeyMapping = {
	SkipBack : new KeyDescriptor( 119, true, true ),
	PlayPause : new KeyDescriptor( 120, true, true ),
	HalfSpeed : new KeyDescriptor( 121, true, true ),
	NormalSpeed : new KeyDescriptor( 122, true, true ),
	DoubleSpeed : new KeyDescriptor( 123, true, true )
};
// 119-123: F8-F12 

var KeyFunctions = {
	PlayPause : function( panel ) {
		panel.playPause();
	},
	HalfSpeed : function( panel ) {
		panel.setSpeed( 0.5 );
	},
	NormalSpeed : function( panel ) {
		panel.setSpeed( 1 );
	},
	DoubleSpeed : function( panel ) {
		panel.setSpeed( 2 );
	},
	SkipBack : function( panel ) {
		panel.skipBack( 5 );
	}
};

function registerDictationPanelKeyListener() {
	$( document ).keyup( function( e ) {
		var panel = getDictationPanel();
		if ( panel ) {
			var op;
			for ( op in KeyMapping ) {
				if ( KeyMapping[ op ].matches( e ) ) {
					e.preventDefault();
					KeyFunctions[ op ]( panel );
					return;
				}
			}
		}
	} );
}

function getDictationPanel( soundref ) {
	if ( !soundref ) {
		soundref = currentDictationPanelId;
	}
	if ( typeof ( dictationPanels[ soundref ] ) == 'undefined' ) {
		dictationPanels[ soundref ] = new DictationPanel( soundref );
	}
	return dictationPanels[ soundref ];
}

function createButton( panel, img, func, title ) {
	var res = $( '<img src="/icons/svg/image?class=' + img + '">' );
	res.css( 'cursor', 'pointer' );
	res.css( 'height', '32px' );
	res.css( 'width', '32px' );
	res.data( 'panel', panel );
	if ( title ) {
		res.attr( "title", title );
	}
	res.click( func );
	res.appendTo( panel.buttonBar );
	return res;
}

function createSpacer( panel, width ) {
	var res = $( '<img src="/icons/svg/image?class=spacer">' );
	res.css( 'height', '32px' );
	res.css( 'width', width );
	res.appendTo( panel.buttonBar );
	return res;
}

function createSpeedBtn( panel, img, speed, title ) {

	var res = createButton( panel, img, function() {
		$( this ).data( 'panel' ).setSpeed( $( this ).data( "speed" ) );
	}, title );
	res.data( 'speed', speed );
	panel.buttons.push( res );
	return res;
}

function DictationPanel( soundref ) {

	this.soundref = soundref;

	this.hoverDiv = null;

	this.buttons = new Array();

	this.togglePanel = function( event ) {
		if ( this.initIfNeeded() ) {
			return;
		}
		if ( this.hoverDiv.isShown() ) {
			this.audioControl.pause();
			this.hoverDiv.hide();
		}
		else {
			this.hoverDiv.show();
			if ( event ) {
				this.hoverDiv.position( getEventX( event ), getEventY( event ) );
			}
			this.audioControl.play();
			currentDictationPanelId = this.soundref;
		}
	}

	this.initIfNeeded = function( keepHidden ) {
		if ( this.hoverDiv == null ) {
			this.hoverDiv = new DraggableHoverDiv( false, false ); // non-modal, non-closeable
			this.audioControl = document.createElement( "audio" );
			this.audioControl.id = 'audio-player_' + this.soundref;
			this.audioControl.controls = 'controls';
			this.audioControl.src = this.soundref;
			this.audioControl.type = 'audio/mpeg';
			this.hoverDiv.contentDiv.appendChild( this.audioControl );

			this.buttonBar = $( '<div/>' );
			this.buttonBar.css( 'position', 'absolute' );
			this.buttonBar.css( 'top', '1px' );
			this.buttonBar.css( 'right', '1px' );
			this.buttonBar.css( 'height', '32px' );
			this.buttonBar.appendTo( this.hoverDiv.hoverDiv );

			createButton( this, "skip_back", function() {
				$( this ).data( 'panel' ).skipBack( 10 );
			}, DictationLangResMap.DICTATION_SKIP_BACK );

			createSpacer( this, '16px' );

			createSpeedBtn( this, "cnext", 0.5,
					DictationLangResMap.DICTATION_HALF_SPEED );
			createSpeedBtn( this, "aplay", 1,
					DictationLangResMap.DICTATION_NORMAL_SPEED );
			createSpeedBtn( this, "ffwd", 2,
					DictationLangResMap.DICTATION_DOUBLE_SPEED );

			createSpacer( this, '16px' );

			createButton( this, "close_new", function() {
				$( this ).data( 'panel' ).togglePanel();
			}, DictationLangResMap.DICTATION_CLOSE );

			this.setSpeed( 1 );
			if ( !keepHidden ) {
				this.togglePanel();
			}
			return true;
		}
		return false;
	}

	this.getAudioControl = function( dontShow ) {
		this.initIfNeeded( dontShow );
		return this.audioControl;
	}

	this.playPause = function() {
		if ( this.initIfNeeded() ) {
			return;
		}
		if ( this.audioControl.paused ) {
			this.audioControl.play();
		}
		else {
			this.audioControl.pause();
		}
	}

	this.skipBack = function( amount ) {
		if ( this.initIfNeeded() ) {
			return;
		}
		var pos = this.audioControl.currentTime;
		pos -= amount;
		if ( pos < 0 ) {
			pos = 0;
		}
		this.audioControl.currentTime = pos;
	}

	this.setSpeed = function( speed ) {
		this.initIfNeeded();
		this.audioControl.playbackRate = speed;
		for ( i = 0; i < this.buttons.length; i++ ) {
			var btn = this.buttons[ i ];
			if ( btn.data( "speed" ) == speed ) {
				// selected button
				btn.css( 'opacity', '1' );
			}
			else {
				btn.css( 'opacity', '0.3' );
			}
		}
	}
}