#!/bin/bash
#
# Component: cases
#
# Description:
#  Retrieves dictation wave file from $REPOSITORY_CACHE.
#

. $DOCUMENT_ROOT/../etc/setenv.sh

# $PATH_INFO looks like "/styiuid/xxx.wav"
study=`echo "$PATH_INFO" | awk '{split($0,a,"/"); print a[2]}'`
file=`echo "$PATH_INFO" | awk '{split($0,a,"/"); print a[3]}'`
studydir=`$REPOSITORYHANDLER_HOME/bin/locateResource $REPOSITORY_CACHE $study`
if [ $? -ne 0 ] ; then
	echo "Content-type: text/html"
	echo "Status: 403"
	echo
	echo "File not found: $PATH_INFO"
	exit 0
fi	

FILE="$studydir/$file"

CONTENT_TYPE="audio/x-wav"

# check and convert wav to mp3 if needed
if [ ! -e ${FILE} -a ${FILE: -4} == ".mp3" ] ; then
	CONTENT_TYPE="audio/mpeg"
	WAVFILE="${FILE%.*}"
	if [ -f ${WAVFILE} ] ; then
		ffmpeg -i ${WAVFILE} ${FILE} >/dev/null 2>>${FILE}.err
	fi
fi

if [ -f ${FILE} ] ; then
	echo "Content-type: ${CONTENT_TYPE}"
	length=`wc -c ${FILE} | sed 's/^ *\([^ ]*\) .*$/\1/'`
	echo "Accept-Ranges: bytes"
	if [[ $HTTP_RANGE == bytes=* ]] ; then
		#http range gets to us like this: HTTP_RANGE=bytes=0-20000

		from=`echo "$HTTP_RANGE"|awk -F"=" '{print $2}' | awk -F"-" '{print $1}'`
		[ "$from" == "" ] && from="0"
		
		to=`echo "$HTTP_RANGE"|awk -F"=" '{print $2}' | awk -F"-" '{print $2}'`
		[ "$to" == "" ] && to=$((length-1))
		
		echo "Content-length: $((to-from+1))"
		echo "Content-Range: bytes $from-$to/$length"
		echo "Status: 206"
		echo
		dd if=${FILE} skip=$from count=$((to-from+1)) bs=1 2>/dev/null
	else
		echo "Content-length: $length"
		echo "Status: 200"
		echo
		cat ${FILE}
	fi
else
	echo "Content-type: text/html"
	echo "Status: 403"
	echo
	echo "File not found: $PATH_INFO"
fi
