<%--
 ** Component: cases
 **
 ** WEB interface for resetting study. This JSP is only used in mixed-mode when invoked by V7 ancestor server in V7 style.
 **
 ** Reset operations:
 **
 **  - Cache:
 **      clear cache directory of the study/studies
 **  - Process:
 **      clear processed directory of the study/studies
 **  - Index:
 **      reindex DICOM files of the study/studies. Reindex will remove
 **      non existing DICOM objects from the database and will start
 **      storescp_reg.sh to reindex existing DICOM objects
 **  - Full:
 **      = Cache + Process + Index
 **
 ** Parameters:
 **
 **  @param cGr       Study Instance UID of the study to be reseted
 **  @param cBack     URL to go back after reset
 **  @param cOp       Reset operation: Cache, Process, Index, Full
 **
 ** $Author$
 ** $Date$
 **
 *
 *  RCS File Name    : $RCSfile$
 *  RCS Revision     : $Revision$
 *  Locked by        : $Locker$
 *  Symbolic name    : $Name$
 *  State            : $State$
--%><%@page import="com.erad.pacs.modules.gwt.exception.usermanager.SessionException"
%><%@page import="com.erad.pacs.connector.Connector"
%><%@page import="com.erad.pacs.services.usermanager.client.interfaces.UserManagerFacade"
%><%@page import="com.erad.pacs.services.usermanager.client.Session"
%><%@page import="imagemed.repohandler.Repository"
%><%@page import="imagemed.repohandler.RepositoryHandler"
%><%@
	page errorPage ="../main/exceptionHandler.jsp"

%><%@
	page import="imagemed.util.*,imagemed.imsession.*,imagemed.jsputil.*,java.sql.*,java.io.*,java.util.*,java.nio.file.*"

%><%!

	static final int SORT_NONE = 0;
	static final int SORT_REGULAR = 1;
	static final int SORT_REVERSE = 2;

	private class FileSorter implements Comparator<String> {

		int m_sortMode;
		String m_directory;

		public FileSorter( String dirName ) {
			m_directory = dirName;
			m_sortMode = SORT_NONE;
		}

		public FileSorter( String dirName, int sortMode ) {
			m_directory = dirName;
			m_sortMode = sortMode;
		}

		@SuppressWarnings ( "all" )
		public int compare( String o1, String o2 ) {
			int res = compareP( o1, o2 );
			return m_sortMode == SORT_REVERSE ? 0 - res : res;
		}


		/**
		 * Compares to filenames:
		 *
		 * PbR &lt; SRt &lt; PbD &lt; others
		 */
		private int compareP( String f1, String f2 ) {
			if ( f1 == null ) {
				if ( f2 == null )
					return 0;
				return -1;
			}
			if ( f2 == null )
				return 1;

			boolean isF1PbR = f1.startsWith( "PbR." );
			boolean isF2PbR = f2.startsWith( "PbR." );
			boolean isF1PbD = f1.startsWith( "PbD." );
			boolean isF2PbD = f2.startsWith( "PbD." );
			boolean isF1SRt = f1.startsWith( "SRt." );
			boolean isF2SRt = f2.startsWith( "SRt." );

			if ( isF1PbR ) {
				if ( isF2PbR )
					return f1.compareTo( f2 );
				return -1;
			}
			if ( isF2PbR )
				return 1;

			if ( isF1SRt ) {
				if ( isF2SRt )
					return f1.compareTo( f2 );
				return -1;
			}
			if ( isF2SRt )
				return 1;

			if ( isF1PbD ) {
				if ( isF2PbD )
					return f1.compareTo( f2 );
				return -1;
			}
			if ( isF2PbD )
				return 1;

			return f1.compareTo( f2 );
		}

		public String[] list( String directoryName ) {
			DirectoryStream.Filter<Path> filter = new DirectoryStream.Filter<Path>() {

				@Override
				public boolean accept( Path path ) throws IOException {
					Path fileNamePath = ( ( path == null ) ? null : path.getFileName() );
					if ( fileNamePath != null ) {
						String name = fileNamePath.toString();
						return !name.startsWith( "." ) && !name.startsWith( "storestate" );
					}

					return false;
				}
			};

			String f[] = Helper.getDirectoryContentAsStringArray( directoryName, filter );
			if ( m_sortMode != SORT_NONE )
				Arrays.sort( f, this );
			return f;
		}

		public String[] list() {
			return list( m_directory );
		}
	}



	/**
	 * Collects objects to be removed from database before reindexing the study
	 *
	 * @param study   Study Instance UID of the study to be reindexed
	 *
	 * @return   Properties of Filename, SOP I UID of objects to be removed from the database
	 */
	Properties getRemoveObjects( String study ) {
		Properties prp = new Properties();
		try {
			ResultSet rs = IMEnv.getDB().prepareStatement(
					"select FNAME, SOPIUID from " + IMEnvBase.getEnv( "OBJECTTBL" ) + " where STYIUID='" + study + "'" ).executeQuery();

			while ( rs.next() )
				prp.setProperty( rs.getString( "FNAME" ), rs.getString( "SOPIUID" ) );
			rs.close();

			String files[] = ( new FileSorter( RepositoryHandler.getLocation( Repository.DICOM, study ) ) ).list();
			for ( int i = 0; i < files.length; i++ ) {
				prp.remove( files[ i ] );
			}
		}
		catch ( Throwable th ) {
		}

		return prp;
	}


	/**
	* Removes non existing DICOM objects from database before reindexing the study
	*
	* @param study   Study Instance UID of the study to be reindexed
	*/
	void removeObjectsFromDB( String study ) {
		try {
			Connection db = IMEnv.getDB();

			// Remove all entry from the report table
			db.prepareStatement( "delete from " + IMEnvBase.getEnv( "REPORTTBL" ) + " where SRCSTUDY='" + study + "'" ).executeUpdate();

			Properties prp = getRemoveObjects( study );
			if ( prp == null || prp.isEmpty() )
				return;

			for ( Enumeration<?> e = prp.propertyNames(); e.hasMoreElements(); ) {
				String fname = (String) e.nextElement();
				String sopi = prp.getProperty( fname );

				db.prepareStatement( "delete from " + IMEnvBase.getEnv( "OBJECTTBL" ) + " where STYIUID='" +
						study + "' and SOPIUID='" + sopi + "'" ).executeUpdate();
			}
		}
		catch ( Throwable th ) {
		}
	}
	
%><%-- Input Parameters --%><%
	
	Session is = null;
	UserManagerFacade userManager = Connector.getInstance().get( UserManagerFacade.Caller );
	try {
		is = userManager.getSessitaSessionNoSessParCheck( request, response );
		if ( request.getParameter( "r_no_refresh" ) == null )
			userManager.updateLastSeen( is.getSessionId() );
	}
	catch ( SessionException se ) {
		response.setHeader( "IMAuthenticationInfo", "nosession" );
		return;
	}
	
String parStudy = request.getParameter( "cGr" );
String parBack = request.getParameter( "cBack" );
String parOp = request.getParameter( "cOp" );

if ( null == parBack )
	parBack = request.getHeader( "Referer" );

if ( null == parBack )
	parBack = "javascript:self.history.back();";

String redirHost = RedirectToolbox.getRedirectForStudy( is, parStudy );
if ( redirHost != null ) {
	String redirURL = redirHost + "/cases/reset.jsp?" + RedirectToolbox.getRedirectParams( is, request );
	response.sendRedirect( redirURL );
	return;
}
%><HTML><HEAD><TITLE>Reset Study</title>
<META http-equiv="Refresh" content="3; URL=<%=parBack%>">
<%
imagemed.http.tabs.TabBean.writeStyleOnly( new PrintWriter(out) );
%>
</HEAD>
<BODY style='text-align: center'>
<BR><BR><BR>
<%
String backupid = Helper.longToDateTime( System.currentTimeMillis() ) + "." + Helper.getCounter();
String env[] = new String[ 1 ];
env[ 0 ] = "HOME=" + IMEnvBase.getEnv( "HOME" );

if ( parOp.equals( "Cache" ) || parOp.equals( "Full" ) ) {
	out.println( "Clearing Cache for " + parStudy + "<BR>" );

	String cmd[] = new String[ 6 ];
	cmd[ 0 ] = IMEnvBase.getEnv( "TOOLS_HOME" ) + "/backupdir.sh";
	cmd[ 1 ] = "-r";
	cmd[ 2 ] = "-d";
	cmd[ 3 ] = IMEnvBase.getEnv( "TMP" ) + "/backup_" + backupid + "_cache_" + parStudy;
	cmd[ 4 ] = "-s";
	cmd[ 5 ] = RepositoryHandler.getLocation( Repository.CACHE, parStudy );

	Helper.runCommand( cmd, env );
}
if ( parOp.equals( "Process" ) || parOp.equals( "Full" ) ) {
	out.println( "Clearing Processed for " + parStudy + "<BR>" );

	String cmd[] = new String[ 6 ];
	cmd[ 0 ] = IMEnvBase.getEnv( "TOOLS_HOME" ) + "/backupdir.sh";
	cmd[ 1 ] = "-r";
	cmd[ 2 ] = "-d";
	cmd[ 3 ] = IMEnvBase.getEnv( "TMP" ) + "/backup_" + backupid + "_processed_" + parStudy;
	cmd[ 4 ] = "-s";
	cmd[ 5 ] = RepositoryHandler.getLocation( Repository.PROCESSED, parStudy );

	Helper.runCommand( cmd, env );
}
if ( parOp.equals( "Index" ) || parOp.equals( "Full" ) ) {
	out.println( "Reindexing " + parStudy + "<BR>" );

	removeObjectsFromDB( parStudy );

	String fnames[] = ( new FileSorter( RepositoryHandler.getLocation( Repository.DICOM, parStudy ), SORT_REGULAR ) ).list();

	for ( int j = 0; j < fnames.length; j++ ) {
		out.println( fnames[ j ] + "<BR>" );
	}

	String cmd[] = new String[ 3 ];
	cmd[ 0 ] = IMEnvBase.getEnv( "CASES_HOME" ) + "/reindexStudy.sh";
	cmd[ 1 ] = "-r";
	cmd[ 2 ] = parStudy;

	Helper.runCommand( cmd, null );
}
%>
</BODY>
</HTML>
