#!/bin/sh
#
# Component: cases
#
# Description:
#  Removes study
#
# Before removing study, rmstudy.sh checks if the study is deletable.
# A study is deletable if all the DICOM objects data/dicom/<study> directory is
# registered in the database (Dcobject table) and the storelevel is 'C' for all the
# DICOM objects.
#
# If the study exists on one of the child servers, a reference storestate.rec file
# with ProcessMode=Deleted will remain in the data/dicom/<study> directory.
#
# If this server is the owner of the study, then study ownership will be passed
# to parent (if parent exists) or to one of the childs on the StagedIPs chain.
#
# Usage:
#   rmstudy.sh [-f] [-r] <study> [<folder>]
#      -f     : Force deleting study even if not yet sent to 3rd party archive
#      -r     : Keep reports (-f and folder will be ignored)
#      study  : Study Instance UID of the study to be deleted
#      folder : remove study from this folder (for studies created by copy to folder)
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

. $HOME/etc/setenv.sh

study=""
folder=""
isForced="false"
keepReports="false"
ERRMSG=""

while getopts 'fr' OPT; do
	case $OPT in
	f) isForced="true" ;;
	r) keepReports="true" ;;
	*) ERRMSG="Invalid Option: $OPT" ;;
	esac
done

shift $(($OPTIND-1))

study=$1
folder=$2

if [ -z "$study" ] ; then
  echo "Missing parameter"
  echo "usage:"
  echo "   rmstudy [-f] [-r] study [folder]"
  exit 1
fi

[ -f "$CONF/taskd.rc" ] && . $CONF/taskd.rc

params="-s \"$study\""
[ "true" = "$isForced" ] && params="-f $params"
[ "true" = "$keepReports" ] && params="-r $params"
[ -n "$folder" ] && params="$params -F \"$folder\""
[ -n "$LOG_USER" ] && params="$params -u \"$LOG_USER\""



# echo "0 ${CASES_HOME#$HOME/component/}.RmStudy $params" 
$TASKD_HOME/runjavacmd -c "0 ${CASES_HOME#$HOME/component/}.RmStudy $params" -p $JAVA_SERVER_PORT

exit $?

