#!/bin/bash

if [[ "$#" -ne 2 ]]; then
    printf "Usage: restoreConfig.sh configToReplaceWith configToReplace\nExample:\nreplaceConfig.sh /home/medsrv/data/users/testuser/config.xml.save_20200101000000000.xml /home/medsrv/data/users/testuser/config.xml\n"
    exit 2
fi

if [[ -z "${SERVERIP}" ]]; then
    echo "Environemnt variable SERVERIP does not exist or empty"
    exit 2
fi

if [[ -z "${SERVERNAME}" ]]; then
    echo "Environemnt variable SERVERNAME does not exist or empty"
    exit 2
fi

echo "Local server IP:${SERVERIP}  Name:${SERVERNAME}"
#ip4=$(/sbin/ip -o -4 addr list eth0 | awk '{print $4}' | cut -d/ -f1)
#echo local server "IP (eth0): $ip4"

#great sed tutorial: https://www.grymoire.com/Unix/Sed.html

printf -v date '%(%Y%m%d_%H%M%S)T' -1
#echo "Current date and time: ${date}"

jcfgPath="/home/medsrv/var/conf/dotcom.jcfg"
#jcfgPath="./remotedotcom.jcfg"
configToReplace=$2
configToReplaceWith=$1
configBackup=${configToReplace}.${date}.replaced


echo "Replace configuration $configToReplace with local backup $configToReplaceWith on all dotcom devices..."

# collect host names ad ip addresses of hosts from jcfg; each device is separated by ; and name and ip is separated by ,
hostTags=$(sed -n '
/\s*<DICOMSettings>/,/\s*<\/DICOMSettings>/ {
    s/\s*<SERVERIP>\(.*\)<\/SERVERIP>/\1,/ p
    s/\s*<HOST>\(.*\)<\/HOST>/\1,/ p
}
/\s*<\/DICOMSettings>/ a\;
' < $jcfgPath)

# remove whitespaces and newlines
hostTags=$(echo "$hostTags" | sed 's/[[:space:]]*//g' | tr -d '\n')

# loop through devices separated by ;
for hostInfoList in ${hostTags//;/ }
do
    # there can be a hostname and an ip for a device separated by ,; I don't know the order or which of them exists but I don't care, I can use any of them so lets use the first data.
    serverInfo1=$(echo "$hostInfoList" | awk -F, '{print $1}') # IP or hostname
    serverInfo2=$(echo "$hostInfoList" | awk -F, '{print $2}') # IP or hostname or empty
    # if any info equals to the servername or server ip then it is the local machine
    if [[ "${SERVERIP}" == "$serverInfo1" ]] || [[ "${SERVERNAME}" == "$serverInfo2" || "${SERVERIP}" == "$serverInfo2" ]] || [[ "${SERVERNAME}" == "$serverInfo1" ]]; then
	echo "Local Device $serverInfo1: rename original config file on LOCAL server ${configToReplace} to ${configBackup}"
	# ssh command in double quotes resolves variables here and send them to the host while in single quotes they do not get resolved
	mv ${configToReplace} ${configBackup}
	echo "Local Device $serverInfo1: copy LOCAL configuration backup file $configToReplaceWith to ${configToReplace}"
	cp ${configToReplaceWith} ${configToReplace}
    else
	echo "Remote Device $serverInfo1: rename original config file on REMOTE server ${configToReplace} to ${configBackup}"
	# ssh command in double quotes resolves variables here and send them to the host while in single quotes they do not get resolved
	ssh -A "medsrv@$serverInfo1" "mv ${configToReplace} ${configBackup}"
	echo "Remote Device $serverInfo1: copy LOCAL configuration backup file $configToReplaceWith to REMOTE server $serverInfo1:${configToReplace}"
	scp ${configToReplaceWith} medsrv@$serverInfo1:${configToReplace}
    fi
done