package imagemed.layout.validator;

import java.text.Format;
import java.util.GregorianCalendar;

import com.erad.pacs.services.usermanager.client.Session;

import imagemed.types.enumeration.IMTypehs;
import imagemed.util.LocalizedMessage;

public class DefaultschOrderValidator extends DBValidator {

	@Override
	protected void preValidate( Session session ) {
		String block = getValue( session, "0ZBL" );
		boolean isBlock = "yes".equals( block );

		//Patient and Procedure must be present
		if ( !isBlock ) {
			setMandatoryError( "0ZPI" );
			setMandatoryError( "0ZRI" );

			//Referrer ID should be present
			setMandatoryWarning( "0YRI" );
		}

		//Acc. No. should be present, but its auto-generated 
		String orderStateStr = getValue( session, "0ZST" );
		// Commented out
		// for more information CBT #1917
		/*int orderState = IMTypeos.INTSTATE_ORDERED;
		//for new Orders this will be the default state (not needed in normal case)
		try {
			orderState = Integer.parseInt( orderStateStr );
		}
		catch ( Throwable th ) {
		}*/

		//DoB -> Minor calculation (all the time, because scheduling may be change it) 
		for ( RecordIterator patient = getIterator( m_schPatientTable ); patient.next( session ); ) {
			String birthDate = patient.getValue( session, "0BDT" );
			if ( !"".equals( birthDate ) && !"_".equals( birthDate ) ) {
				try {
					GregorianCalendar patCal = getCalendarFromDate( birthDate );
					patCal.add( GregorianCalendar.YEAR, 18 );
					GregorianCalendar compareCal = getOrderCalendar( /*orderState*/ ); // Commented out 'orderState' for more information CBT #1917
					//Scheduled start time, if appropriate, else now

					//birth + 18 < order time
					if ( patCal.before( compareCal ) ) {
						setValue( "0ZMN", "no" );
					}
					else {
						setValue( "0ZMN", "yes" );
					}
				}
				catch ( Exception e ) {
				}
			}
		}

		// Laterality
		for ( RecordIterator proc = getIterator( m_schProcedureTable ); proc.next( session ); ) {
			String procLaterality = proc.getValue( session, "0HLA" );
			if ( "Unilateral".equals( procLaterality ) ) {
				setMandatoryWarning( "0ZLA" );
			}
		}

		//Refresh - Set fields
		String refreshingColid = getRefreshingColid();
		String refreshingType = getRefreshType();

		if ( REFRESHTYPE_FIELDCHANGE.equals( refreshingType ) ) {
			//Procedure selected 
			if ( "0ZRI".equals( refreshingColid ) ) {
				for ( RecordIterator proc = getIterator( m_schProcedureTable ); proc.next( session ); ) {
					// Amount is Cost from Procedure
					String cost = proc.getValue( session, "0HCO" );
					if ( cost != null ) {
						setValue( "0ZAM", cost );
					}
				}
			}

			//Procedure or Insurer cost changes -> Balance counting from Procedure cost - Insurer costs
			if ( "0ZRI".equals( refreshingColid ) || "0QPE".equals( refreshingColid ) ) {
				String amount = getValue( session, "0ZAM" );
				if ( amount != null ) {
					try {
						float floatBalance = Float.parseFloat( amount );
						//						int intBalance = Integer.parseInt( amount );
						for ( RecordIterator orderInsurer = getIterator( m_schOrderInsurerTable ); orderInsurer.next( session ); ) {
							String insCost = orderInsurer.getValue( session, "0QPE" );
							if ( insCost != null ) {
								try {
									float floatInsCost = Float.parseFloat( insCost );
									//									int intInsCost = Integer.parseInt( insCost );
									floatBalance -= floatInsCost;
								}
								catch ( Throwable th ) {
								}
							}
						}
						if ( floatBalance < 0 ) {
							floatBalance = 0;
						}
						setValue( "0ZBA", "" + floatBalance );
					}
					catch ( Throwable th ) {
					}
				}
			}

			RecordIterator orderInsurer = null;
			boolean insurerCopy = false;
			//Insurer changes or check-in -> Copy fields from Patient-Insurer to Order-Insurer
			if ( "0QPI".equals( refreshingColid ) ) {
				insurerCopy = true;
				orderInsurer = getRefreshingRecords();
				//Insurer changes, change only what's changed
			}
			else if ( "0ZCS".equals( refreshingColid ) && IMTypehs.STATE_CHECKEDIN.equals( getValue( session, "0ZCS" ) ) ) {
				insurerCopy = true;
				orderInsurer = getIterator( m_schOrderInsurerTable );
				//Check-in, copy all
			}

			if ( insurerCopy && orderInsurer != null && !orderInsurer.isEmpty( session ) ) {
				while ( orderInsurer.next( session ) ) {
					String patientInsurerID = orderInsurer.getValue( session, "0QPI" );
					if ( patientInsurerID != null ) {
						for ( RecordIterator patient = getIterator( m_schPatientTable ); patient.next( session ); ) { //there will be 1
							for ( RecordIterator patientInsurer = patient.getIterator( m_schPatientInsurerTable ); patientInsurer.next( session ); ) {
								String recordID = patientInsurer.getValue( session, "0BID" );
								if ( patientInsurerID.equals( recordID ) ) {
									String[][] copyFromPatientInsurerToOrderInsurer = {
											{ "0BPO", "0QPO" }, { "0BED", "0QED" }, { "0BTD", "0QTD" }, { "0BGR", "0QGR" },
											{ "0BAN", "0QAN" }, { "0BAS", "0QAS" }, { "0BAB", "0QAB" }, { "0BAR", "0QAR" },
											{ "0BAE", "0QAE" }, { "0BA2", "0QA2" }, { "0BCI", "0QCI" }, { "0BST", "0QST" },
											{ "0BZI", "0QZI" }, { "0BPH", "0QPH" } };
									copyValues( session, copyFromPatientInsurerToOrderInsurer, patientInsurer, orderInsurer );
								}
							}
						}
					}
				}
			}

			//Patient changed or Check In: copy fields from Patient to Order
			if ( "0ZPI".equals( refreshingColid ) ||
					( "0ZCS".equals( refreshingColid ) && IMTypehs.STATE_CHECKEDIN.equals( getValue( session, "0ZCS" ) ) ) ) {
				String[][] copyFromPatientToOrder = { { "0PPR", "0ZPR" }, { "0PWA", "0ZWA" }, { "0PBW", "0ZBW" },
						{ "0PAC", "0ZPA" }, { "0MET", "0ZME" }, { "0HOC", "0ZHC" }, { "0HOB", "0ZHB" }, { "0WWM", "0ZWM" },
						{ "0PSU", "0ZPS" }, { "0CLA", "0ZCL" }, { "0DIA", "0ZDI" }, { "0MMY", "0ZMM" }, { "0PIV", "0ZPV" },
						{ "0PGL", "0ZGL" }, { "0PAH", "0ZAH" }, { "0PKP", "0ZKP" }, { "0PIP", "0ZIP" }, { "0PTB", "0ZTB" },
						{ "0PGB", "0ZGB" }, { "0PIE", "0ZII" }, { "0PWV", "0ZWV" }, { "0PWN", "0ZWN" }, { "0PWP", "0ZWP" },
						{ "0PWI", "0ZWI" }, { "0PAN", "0ZAC" }, { "0PAP", "0ZAP" }, { "0PAT", "0ZAD" } };
				for ( RecordIterator patient = getIterator( m_schPatientTable ); patient.next( session ); ) {
					copyValues( session, copyFromPatientToOrder, patient, null );
				}
			}

		}

	}


	@Override
	public void validate( Session session ) {
		//Core stuff
		String block = getValue( session, "0ZBL" );
		boolean isBlock = "yes".equals( block );

		String orderStateStr = getValue( session, "0ZST" );
		String oldOrderStateStr = getDBValue( session, "0ZST" );

		// Commented out
		// for more information CBT #1917
		/*int orderState = IMTypeos.INTSTATE_ORDERED;
		int oldOrderState = IMTypeos.INTSTATE_ORDERED;
		//for new Orders this will be the default state (not needed in normal case)
		try {
			orderState = Integer.parseInt( orderStateStr );
			oldOrderState = Integer.parseInt( oldOrderStateStr );
		}
		catch ( Throwable th ) {
		}*/

		//Acc. No. should be unique
		if ( !isBlock ) {
			shouldBeUnique( session, "0ZAN", m_orderPage );

			RecordIterator orderRefPhys = getIterator( m_schOrderReferrerTable );
			if ( orderRefPhys.isEmpty( session ) ) {
				m_warnings.add( LocalizedMessage.getSimpleMessage( "DEFAULT_SCH_ORDER_VALIDATOR_AT_LEAST_ONE_REF" ) );
			}
		}

		//If pre-certification is required, does a pre-certification number exist?
		//for all Insurance companies connected to this order
		for ( RecordIterator orderInsurer = getIterator( m_schOrderInsurerTable ); orderInsurer.next( session ); ) {
			String patientInsurerID = orderInsurer.getValue( session, "0QPI" );
			if ( patientInsurerID != null ) {
				for ( RecordIterator patient = getIterator( m_schPatientTable ); patient.next( session ); ) { //there will be 1
					for ( RecordIterator patientInsurer = patient.getIterator( m_schPatientInsurerTable ); patientInsurer.next( session ); ) {
						String recordID = patientInsurer.getValue( session, "0BID" );
						if ( patientInsurerID.equals( recordID ) ) {
							for ( RecordIterator insurer = patientInsurer.getIterator( m_schInsurerTable ); insurer.next( session ); ) {
								if ( "yes".equals( insurer.getValue( session, "0IPR" ) ) &&
										"".equals( orderInsurer.getValue( session, "0QCN" ) ) ) {
									String insName = insurer.getValue( session, "0INA" );
									m_warnings.add( LocalizedMessage.getCompoundMessage( "DEFAULT_SCH_ORDER_VALIDATOR_INSURANCE_COMPANY_CERTIFICATE", new Format[]{ null }, new Object[]{ insName }, false ) );
								}
							}
						}
					}
				}
			}
		}

		//If patient pregnant?
		// and waiver form not scanned
		if ( "yes".equals( getValue( session, "0ZPR" ) ) ) {
			shouldHaveAttachment( LocalizedMessage.getSimpleMessage( "DEFAULT_SCH_ORDER_VALIDATOR_PATIENT_IS_PREGNANT" ), "order", "Waiver Form" );
		}

		//Patient and Procedure-related
		String contrast = getValue( session, "0ZA1" );
		boolean contrastUsed = !"".equals( contrast ) && !"---".equals( contrast );
		for ( RecordIterator patient = getIterator( m_schPatientTable ); patient.next( session ); ) {
			//If patient is not female, no pregnant & last menstrual
			if ( !"F".equals( patient.getValue( session, "0SEX" ) ) ) {
				if ( "yes".equals( getValue( session, "0ZPR" ) ) ) {
					m_warnings.add( LocalizedMessage.getSimpleMessage( "DEFAULT_SCH_ORDER_VALIDATOR_NOT_FEMALE_PREGNANCY_INDICATED" ) );
				}
				if ( !"".equals( getValue( session, "0ZLM" ) ) ) {
					m_warnings.add( LocalizedMessage.getSimpleMessage( "DEFAULT_SCH_ORDER_VALIDATOR_NOT_FEMALE_MENSTRUAL_PERIOD_INDICATED" ) );
				}
			}


			for ( RecordIterator procedure = getIterator( m_schProcedureTable ); procedure.next( session ); ) {
				String modality = procedure.getValue( session, "0HTY" );
				//If contrast is being used and patient has any serious allergies?

				//Kidney problems + Contrast
				if ( contrastUsed &&
						"yes".equals( getValue( session, "0ZKP" ) ) ) {
					m_warnings.add( LocalizedMessage.getSimpleMessage( "DEFAULT_SCH_ORDER_VALIDATOR_CONTRAST_AGENT_KIDNEY_PROBLEMS" ) );
				}

				//If patient is scheduling an MR
				if ( "MR".equals( modality ) ) {
					String MR = LocalizedMessage.getSimpleMessage( "DEFAULT_SCH_ORDER_VALIDATOR_MR_PROCEDURE_SCHEDULED" );
					//Does patient have a pacemaker?
					if ( "yes".equals( getValue( session, "0ZPA" ) ) ) {
						m_warnings.add( LocalizedMessage.getCompoundMessage( "DEFAULT_SCH_ORDER_VALIDATOR_MR_HAS_PACEMAKER", new Format[]{ null }, new Object[]{ MR }, false ) );
					}
					//Does patient have metal implants in the body?
					if ( "yes".equals( getValue( session, "0ZME" ) ) ) {
						m_warnings.add( LocalizedMessage.getCompoundMessage( "DEFAULT_SCH_ORDER_VALIDATOR_MR_HAS_METAL_IMPLANTS", new Format[]{ null }, new Object[]{ MR }, false ) );
					}
					//Does patient work with metal?
					if ( "yes".equals( getValue( session, "0ZWM" ) ) ) {
						m_warnings.add( LocalizedMessage.getCompoundMessage( "DEFAULT_SCH_ORDER_VALIDATOR_MR_WORKS_WITH_METAL", new Format[]{ null }, new Object[]{ MR }, false ) );
					}
					//Does patient have a history of cancer?
					if ( "yes".equals( getValue( session, "0ZHC" ) ) ) {
						m_warnings.add( LocalizedMessage.getCompoundMessage( "DEFAULT_SCH_ORDER_VALIDATOR_MR_HAS_CANCER_HISTORY", new Format[]{ null }, new Object[]{ MR }, false ) );
					}
					//Does patient have a brain tumor?
					if ( "yes".equals( getValue( session, "0ZHB" ) ) ) {
						m_warnings.add( LocalizedMessage.getCompoundMessage( "DEFAULT_SCH_ORDER_VALIDATOR_MR_HAS_BRAIN_TUMOR_HISTORY", new Format[]{ null }, new Object[]{ MR }, false ) );
					}
					//Has patient had surgeries? 
					if ( "yes".equals( getValue( session, "0ZPS" ) ) ) {
						m_warnings.add( LocalizedMessage.getCompoundMessage( "DEFAULT_SCH_ORDER_VALIDATOR_MR_HAS_SURGERIES", new Format[]{ null }, new Object[]{ MR }, false ) );
					}
					//Asthma/Hay Fever with Contrast
					if ( contrastUsed &&
							"yes".equals( getValue( session, "0ZAH" ) ) ) {
						m_warnings.add( LocalizedMessage.getCompoundMessage( "DEFAULT_SCH_ORDER_VALIDATOR_MR_HAS_ASTHMA_HEY_FEVER", new Format[]{ null }, new Object[]{ MR }, false ) );
					}
				}
				//if patient is scheduling a CT, 
				else if ( "CT".equals( modality ) ) {
					String CT = LocalizedMessage.getSimpleMessage( "DEFAULT_SCH_ORDER_VALIDATOR_CT_PROCEDURE_SCHEDULED" );
					//is the patient diabetic?
					if ( "yes".equals( getValue( session, "0ZDI" ) ) ) {
						m_warnings.add( LocalizedMessage.getCompoundMessage( "DEFAULT_SCH_ORDER_VALIDATOR_CT_IS_DIABETIC", new Format[]{ null }, new Object[]{ CT }, false ) );
					}
					//Does the patient have multiple myeloma?
					if ( "yes".equals( getValue( session, "0ZMM" ) ) ) {
						m_warnings.add( LocalizedMessage.getCompoundMessage( "DEFAULT_SCH_ORDER_VALIDATOR_CT_HAS_MULTIPLE_MYELOMA", new Format[]{ null }, new Object[]{ CT }, false ) );
					}
					//Has the patient had a problem with I.V. contrast dye prevention?
					if ( "yes".equals( getValue( session, "0ZPV" ) ) ) {
						m_warnings.add( LocalizedMessage.getCompoundMessage( "DEFAULT_SCH_ORDER_VALIDATOR_CT_HAD_IV_CONTRAST_PROBLEM", new Format[]{ null }, new Object[]{ CT }, false ) );
					}
					//Takes Glucophage and contrast is used?
					if ( contrastUsed &&
							"yes".equals( getValue( session, "0ZGL" ) ) ) {
						m_warnings.add( LocalizedMessage.getCompoundMessage( "DEFAULT_SCH_ORDER_VALIDATOR_CT_TAKES_GLUCOPHAGE", new Format[]{ null }, new Object[]{ CT }, false ) );
					}
					//Has blood work been completed on patients over 50 years old?
					if ( "yes".equals( getValue( session, "0ZBW" ) ) ) {
						String birthDate = patient.getValue( session, "0BDT" );
						if ( !"".equals( birthDate ) ) {
							try {
								GregorianCalendar patCal = getCalendarFromDate( birthDate );
								patCal.add( GregorianCalendar.YEAR, 50 );
								GregorianCalendar compareCal = getOrderCalendar( /*orderState*/ ); // Commented out 'orderState' for more information CBT #1917
								//Scheduled start time, if appropriate, else now

								//birth + 50 < scheduled/now
								if ( patCal.before( compareCal ) ) {
									m_warnings.add( LocalizedMessage.getCompoundMessage( "DEFAULT_SCH_ORDER_VALIDATOR_CT_BLOOD_WORK_OVER_50", new Format[]{ null }, new Object[]{ CT }, false ) );
								}
							}
							catch ( Exception e ) {
							}
						}
					}
				}

				//If last exam was less than N days ago, is procedure for an existing medical condition?
				String scheduledTime = getValue( session, "0ZSS" );
				for ( RecordIterator relatedProcedure = procedure.getIterator( m_schProcedureInterRelationTable ); relatedProcedure.next( session ); ) {
					String relatedProcID = relatedProcedure.getValue( session, "0RPC" );
					String relatedType = relatedProcedure.getValue( session, "0RTY" );
					String relatedDays = relatedProcedure.getValue( session, "0RU1" );
					if ( !"".equals( relatedDays ) &&
							!"".equals( relatedProcID ) &&
							!"".equals( scheduledTime ) &&
							"NotPaidInCaseOf".equals( relatedType ) ) {
						String patientID = getValue( session, "0ZPI" );
						shouldNotBeRelatedProcedures( session, patientID, relatedProcID, relatedDays, scheduledTime, LocalizedMessage.getSimpleMessage( "DEFAULT_SCH_ORDER_VALIDATOR_ORDER_MAY_NOT_BE_PAID" ), new String[]{ "0ZAN", "0ZSS" } );
					}
				}
			}
		}

		if ( !isBlock ) {
			//Laterality
			for ( RecordIterator proc = getIterator( m_schProcedureTable ); proc.next( session ); ) {
				String procLaterality = proc.getValue( session, "0HLA" );
				if ( !"Unilateral".equals( procLaterality ) ) {
					String orderLaterality = getValue( session, "0ZLA" );
					if ( orderLaterality != null && !"".equals( orderLaterality ) && !"-".equals( orderLaterality ) ) {
						m_warnings.add( LocalizedMessage.getSimpleMessage( "DEFAULT_SCH_ORDER_VALIDATOR_LATERALITY_INDICATED" ) );
					}
				}
				//Atmeneti! - mandatory warning doesn't work!
				else {
					String orderLaterality = getValue( session, "0ZLA" );
					if ( orderLaterality == null || "".equals( orderLaterality ) || "-".equals( orderLaterality ) ) {
						m_warnings.add( LocalizedMessage.getSimpleMessage( "DEFAULT_SCH_ORDER_VALIDATOR_LATERALITY_NOT_INDICATED" ) );
					}
				}
			}

			//Computing Time Needed Modifier (Sample)
			//TODO: hide this to a method
			//GregorianCalendar cal = new GregorianCalendar( 0, 0, 0, 0, 0 );
			//			if ( "yes".equals( getValue( "0ZPR" ) ) ) {
			//cal.add( GregorianCalendar.MINUTE, 70 );
			//			}
			//setValue( "0ZTI", Helper.calcTimeStr( cal ) );

		}


		//Check in stuff

		// Commented out
		// for more information CBT #1917
		/*if ( orderState >= IMTypeos.INTSTATE_CHECKEDIN ) {
			//Checkin: TR-IN-4.2Confirm insurance eligibility.
			//removed - field is not on the form
			//			String insuranceEligible = getValue( "0ZIE" );
			//			if( !"yes".equals( insuranceEligible ) ) {
			//				m_warnings.add("Insurance eligibility is not confirmed.");
			//			}
		
			for ( RecordIterator patient = getIterator( m_schPatientTable ); patient.next(); ) {
				String birthDate = patient.getValue( "0BDT" );
				String sex = patient.getValue( "0SEX" );
				if ( !"".equals( birthDate ) ) {
					try {
						GregorianCalendar patCal = getCalendarFromDate( birthDate );
						patCal.add( GregorianCalendar.YEAR, 12 );
						GregorianCalendar compareCal = getOrderCalendar( orderState );
		
						//birth + 13 < scheduled/now
						if ( patCal.before( compareCal ) && "F".equals( sex ) ) {
							shouldBeFilled( new String[]{ "0ZPR", "0ZLM" } );
						}
					}
					catch ( Exception e ) {
					}
				}
			}
		}*/


		//Checkout: User shall be notified if patient is scheduled to take films/CD with them


		// Commented out
		// for more information CBT #1917
		/*//Cancel stuff
		if ( orderState == IMTypeos.INTSTATE_CANCELLED ) {
			shouldBeFilled( new String[]{ "0ZRF" } );
		}
		
		//Scheduled Orders in the past
		if ( orderState == IMTypeos.INTSTATE_SCHEDULED ) {
			String schStartTime = getValue( "0ZSS" );
			if ( !"".equals( schStartTime ) &&
					!"_".equals( schStartTime ) ) {
				GregorianCalendar orderCal = getCalendarFromDate( schStartTime );
				orderCal.add( GregorianCalendar.HOUR, 12 );
				//Start + 12 hours < now
				if ( orderCal.before( new GregorianCalendar() ) ) {
					needRights( Rights.Admin.letterString(), LocalizedMessage.getSimpleMessage( "DEFAULT_SCH_ORDER_VALIDATOR_NO_SCHEDULE_RIGHT_PAST_ORDER" ) );
					m_warnings.add( LocalizedMessage.getSimpleMessage( "DEFAULT_SCH_ORDER_VALIDATOR_PAST_SCHEDULE_SCHEDULED_STATUS" ) );
				}
			}
		}*/

		String oldSchStartTime = getDBValue( session, "0ZSS" );
		String oldSchEndTime = getDBValue( session, "0ZSE" );
		String newSchStartTime = getValue( session, "0ZSS" );
		String newSchEndTime = getValue( session, "0ZSE" );

		// Commented out
		// for more information CBT #1917
		//Unscheduling without rights
		/*if ( ( oldOrderState >= IMTypeos.INTSTATE_SCHEDULED ) &&
				( orderState == IMTypeos.INTSTATE_ORDERED ) ) {
			needRights( IMTypeur.SCHEDULER_SCHEDULE_UNSCHEDULE_RIGHT, LocalizedMessage.getSimpleMessage( "DEFAULT_SCH_ORDER_VALIDATOR_NO_UNSCHEDULE_RIGHT" ) );
		}*/

		//Re-Scheduling checked in orders
		if ( !"".equals( oldSchStartTime ) &&
				!"".equals( oldSchEndTime ) &&
				( !oldSchStartTime.equals( newSchStartTime ) ||
						!oldSchEndTime.equals( newSchEndTime ) ) ) {
			// Commented out
			// for more information CBT #1917
			/*if ( orderState >= IMTypeos.INTSTATE_CHECKEDIN ) {
				m_warnings.add( LocalizedMessage.getSimpleMessage( "DEFAULT_SCH_ORDER_VALIDATOR_CHECKED_IN_RESCHEDULED" ) );
			}*/
		}

		// Commented out
		// for more information CBT #1917
		//Unscheduling Checked in Orders
		/*if ( ( oldOrderState >= IMTypeos.INTSTATE_CHECKEDIN ) &&
				( orderState == IMTypeos.INTSTATE_ORDERED ) ) {
			m_warnings.add( LocalizedMessage.getSimpleMessage( "DEFAULT_SCH_ORDER_VALIDATOR_CHECKED_IN_UNSCHEDULED" ) );
		}*/

		// Commented out
		// for more information CBT #1917
		/*if ( ( orderState >= IMTypeos.INTSTATE_SCHEDULED ) &&
				( orderState != IMTypeos.INTSTATE_CANCELLED ) ) {
			String insidePatientID = getValue( "0ZPI" );
			String orderID = getValue( "0ZID" );
			String scheduleStart = getValue( "0ZSS" );
			String scheduleEnd = getValue( "0ZSE" );
			String scheduleStart1 = SchedulerInterface.addToDateTime( scheduleStart, 1 );
			String scheduleEnd1 = SchedulerInterface.addToDateTime( scheduleEnd, -1 );
		
			//Scheduling this Patient to another exam at the same time
			if ( !"".equals( insidePatientID ) ) {
				String warnPatient = LocalizedMessage.getSimpleMessage( "DEFAULT_SCH_ORDER_VALIDATOR_PATIENT_SCHEDULED_SAME_TIME" );
		
				//result1: schedStart < SS < schedEnd
				//scheduled only
				//myself not
				Properties filters = new Properties();
		
				filters.setProperty( "0ZSS", "(" + scheduleStart + "-" + scheduleEnd1 + ")()()" );
				filters.setProperty( "0ZPI", insidePatientID );
				filters.setProperty( "0ZST", IMTypeos.STATE_SCHEDULED + ":" );
				filters.setProperty( "0ZID", "!" + orderID );
				filters.setProperty( "0ZST", IMTypeos.STATE_SCHEDULED + ";" + IMTypeos.STATE_CHECKEDIN + ":" );
		
				DBIterator allOrders = getDBIterator( new String[]{ "0ZAN" }, filters );
				if ( !allOrders.isEmpty() ) {
					m_warnings.add( warnPatient );
				}
				else {
					//result2: SS < schedStart < SE
					filters.setProperty( "0ZSS", "(-" + scheduleStart + ")()()" );
					filters.setProperty( "0ZSE", "(" + scheduleStart1 + "-)()()" );
					allOrders = getDBIterator( new String[]{ "0ZAN" }, filters );
					if ( !allOrders.isEmpty() ) {
						m_warnings.add( warnPatient );
					}
				}
			}
		
			//Scheduling on a Block or on another Order 
			boolean blockFound = false;
			boolean orderFound = false;
			boolean blockedOrderFound = false;
		
			for ( RecordIterator devproc = getIterator( m_schDeviceProcedureTable ); devproc.next(); ) {
				//there will be 1
				String deviceID = devproc.getValue( "0EDI" );
				if ( deviceID != null ) {
		
					//result1: schedStart < SS < schedEnd
					//same device 
					//myself not
					//scheduled or at least checked in
					Properties filters = new Properties();
		
					filters.setProperty( "0ZSS", "(" + scheduleStart + "-" + scheduleEnd1 + ")()()" );
					filters.setProperty( "0EDI", deviceID );
					filters.setProperty( "0ZID", "!" + orderID );
					filters.setProperty( "0ZST", IMTypeos.STATE_SCHEDULED + ";" + IMTypeos.STATE_CHECKEDIN + ":" );
		
					MTDBIterator allOrders = getMTDBIterator( new String[]{ "0ZBL" }, filters );
					while ( allOrders.next() ) {
						if ( isBlock ) {
							if ( !"yes".equals( allOrders.getValue( "0ZBL" ) ) ) {
								blockedOrderFound = true;
								break;
							}
						}
						else {
							if ( "yes".equals( allOrders.getValue( "0ZBL" ) ) ) {
								blockFound = true;
								break;
							}
							else {
								orderFound = true;
							}
						}
					}
					if ( !blockFound && !blockedOrderFound ) {
						//result2: SS < schedStart < SE
						filters.setProperty( "0ZSS", "(-" + scheduleStart + ")()()" );
						filters.setProperty( "0ZSE", "(" + scheduleStart1 + "-)()()" );
						allOrders = getMTDBIterator( new String[]{ "0ZBL" }, filters );
						while ( allOrders.next() ) {
							if ( isBlock ) {
								if ( !"yes".equals( allOrders.getValue( "0ZBL" ) ) ) {
									blockedOrderFound = true;
									break;
								}
							}
							else {
								if ( "yes".equals( allOrders.getValue( "0ZBL" ) ) ) {
									blockFound = true;
									break;
								}
								else {
									orderFound = true;
								}
							}
						}
					}
					if ( blockedOrderFound ) {
						m_warnings.add( LocalizedMessage.getSimpleMessage( "DEFAULT_SCH_ORDER_VALIDATOR_ORDERS_BLOCK_TIME" ) );
					}
					else if ( blockFound ) {
						m_warnings.add( LocalizedMessage.getSimpleMessage( "DEFAULT_SCH_ORDER_VALIDATOR_DEVICE_BLOCKED_ORDER_TIME" ) );
					}
					else if ( orderFound ) {
						needRights( IMTypeur.SCHEDULER_DOUBLE_BOOKING_RIGHT, LocalizedMessage.getSimpleMessage( "DEFAULT_SCH_ORDER_VALIDATOR_NO_RIGHT_DOUBLE_BOOK" ) );
						m_warnings.add( LocalizedMessage.getSimpleMessage( "DEFAULT_SCH_ORDER_VALIDATOR_ANOTHER_ORDER_SAME_TIME" ) );
					}
				}
			}
		}*/
	}


}
