package imagemed.layout.validator;

import java.sql.*;
import java.util.*;

import com.erad.pacs.app.broker.table.TableBrokerFacade;
import com.erad.pacs.connector.Connector;
import com.erad.pacs.modules.gwt.session.right.PacsRight;
import com.erad.pacs.services.usermanager.client.Session;

import imagemed.exception.IMException;
import imagemed.imsession.IMEnv;
import imagemed.jsputil.IMAdminToolbox;
import imagemed.layout.datasource.*;
import imagemed.types.IMType;
import imagemed.types.flag.IMTypeur;
import imagemed.util.*;



public class DefaultuserValidator extends DBValidator {

	@Override
	protected void preValidate( Session session ) {
		setMandatoryError( "USID" );
		setMandatoryError( "UTMZ" );

		String userID = m_rootRec.m_rowID.get( 0 );

		//Refresh - Set fields
		String refreshingType = getRefreshType();


		if ( REFRESHTYPE_FIELDCHANGE.equals( refreshingType ) ) {
			String refreshingColid = getRefreshingColid();

			// default handling
			if ( "ULDU".equals( refreshingColid ) ) {
				//if default is yes now
				RecordIterator actual = getRefreshingRecords();
				while ( actual.next( session ) ) { // there will be 1
					if ( "yes".equals( actual.getValue( session, "ULDU" ) ) ) {

						//default = no, for all 
						for ( RecordIterator userLoc = getIterator( m_userLocationTable ); userLoc.next( session ); ) {
							userLoc.setValue( "ULDU", "no" );
						}

						//ours is put back to yes
						actual.setValue( "ULDU", "yes" );
					}
				}
			}
		}

		// Copy User button
		if ( REFRESHTYPE_BUTTON.equals( refreshingType ) ) {
			if ( "CopyUser".equals( getRefreshingMacroID() ) ) {
				// FIXME: otherID
				//String otherID = m_dataSource.getSession().request.getParameter( "otherID" );
				String otherID = null;
				Helper.DEBUG( "copyuserbutton:" + otherID );
				if ( otherID != null ) {
					try {
						DBLDSEntry other = new DBLDSEntry( (DBLayoutDataSource) m_dataSource );
						other.initFromExisting( session, otherID );

						// copy the needed fields in root record
						Iterator<String> colids = other.getRootRecord().getColidSet().iterator();
						while ( colids.hasNext() ) {
							String field = colids.next();
							if ( !"USID".equals( field ) && !"UPWD".equals( field ) && !"UPW2".equals( field ) &&
									!"UACS".equals( field ) && !"UACW".equals( field ) && !"UCOD".equals( field ) &&
									!"UVER".equals( field ) && !"UNAM".equals( field ) && !"MAIL".equals( field ) && !"UPH1".equals( field ) && !"UPH2".equals( field ) ) {
								setValue( field, other.getRootRecord().getCurrentValue( field, "" ) );
								Helper.DEBUG( "Field " + field + " set to:" + other.getRootRecord().getCurrentValue( field, "" ) );
							}
						}

						// set the locations - will not delete/modify the existing ones, if any
						//detect if there's already a default
						boolean hasDef = false;
						for ( RecordIterator userLoc = getIterator( m_userLocationTable ); userLoc.next( session ); ) {
							if ( "yes".equals( userLoc.getValue( session, "ULDU" ) ) ) {
								hasDef = true;
							}
						}

						List<LDSRecord> otherLocs = other.getRootRecord().getChildList( m_userLocationTable );
						for ( LDSRecord otherLoc : otherLocs ) {
							LDSRecord newRec = m_entry.addRecord( m_userLocationTable, m_rootRec );
							newRec.setValue( "ULUI", userID );
							colids = otherLoc.getColidSet().iterator();
							while ( colids.hasNext() ) {
								String field = colids.next();
								if ( !"ULID".equals( field ) && !"ULUI".equals( field ) &&
										( !hasDef || !"ULDU".equals( field ) ) ) {
									//if there's a default, we leave the new defaults on 'no'
									newRec.setValue( field, otherLoc.getCurrentValue( field, "" ) );
									Helper.DEBUG( "Field " + field + " set to:" + otherLoc.getCurrentValue( field, "" ) );
								}
							}
						}

						// set the practices - will not delete/modify the existing ones, if any
						List<LDSRecord> otherPracs = other.getRootRecord().getChildList( m_userPracticeTable );
						for ( LDSRecord otherPrac : otherPracs ) {
							LDSRecord newRec = m_entry.addRecord( m_userPracticeTable, m_rootRec );
							newRec.setValue( "UPUI", userID );
							colids = otherPrac.getColidSet().iterator();
							while ( colids.hasNext() ) {
								String field = colids.next();
								if ( !"UPID".equals( field ) && !"UPUI".equals( field ) ) {
									newRec.setValue( field, otherPrac.getCurrentValue( field, "" ) );
									Helper.DEBUG( "Field " + field + " set to:" + otherPrac.getCurrentValue( field, "" ) );
								}
							}
						}

						// set the managed groups - will not delete/modify the existing ones, if any
						List<LDSRecord> otherGroups = other.getRootRecord().getChildList( m_userGroupTable );
						for ( LDSRecord otherGroup : otherGroups ) {
							LDSRecord newRec = m_entry.addRecord( m_userGroupTable, m_rootRec );
							newRec.setValue( "UGUI", userID );
							colids = otherGroup.getColidSet().iterator();
							while ( colids.hasNext() ) {
								String field = colids.next();
								if ( !"UGID".equals( field ) && !"UGUI".equals( field ) ) {
									newRec.setValue( field, otherGroup.getCurrentValue( field, "" ) );
									Helper.DEBUG( "Field " + field + " set to:" + otherGroup.getCurrentValue( field, "" ) );
								}
							}
						}

					}
					catch ( Exception e ) {
						Helper.ERRORLOG( "EXCEPTION:" + Helper.throwable2String( e ) );
					}
				}
			}
		}

	}

	@Override
	public void validate( Session session ) {
		TableBrokerFacade tb = Connector.getInstance().get( TableBrokerFacade.Caller );

		Helper.DEBUG( "DefaultuserValidator running" );

		String userID = getValue( session, "USID" );
		if ( !Helper.containsOnly( userID, "0..9a..zA..Z _." ) ) {
			m_errors.add( LocalizedMessage.getSimpleMessage( "USERVALIDATOR_BADID" ) );
			return;
		}

		// we don't use the general method, because Patrick + Jim wants a special message here (CBT 2194)
		//mustBeUnique( "USID", m_userPage, false );
		String actValue = getRootRec().getCurrentValue( "USID", "" );
		if ( !"".equals( actValue ) ) {
			try {
				Statement st = IMEnv.getDB().createStatement();
				String query = "select * from " + getRootRec().m_stable + " where " +
						tb.getDbName( "USID" ) + " = '" + Helper.sqlEncode( actValue ) + "'";
				if ( !getRootRec().isNew() ) {
					query += " AND " + getRootRec().m_IDfieldDbName.get( 0 ) + " != '" + getRootRec().m_rowID.get( 0 ) + "'";
				}
				Helper.DEBUG( "USID query:" + query );
				ResultSet rs = st.executeQuery( query );
				if ( rs != null ) {
					rs.last();
					int numOfResult = rs.getRow();
					rs.beforeFirst();

					if ( numOfResult > 0 ) {
						m_errors.add( LocalizedMessage.getSimpleMessage( "USERVALIDATOR_EXISTS" ) + " " + actValue );
						while ( rs.next() ) {
							String name = rs.getString( tb.getDbName( "UNAM" ) );
							if ( name != null ) {
								IMType type = IMType.getTypeForColid( "UNAM" );
								name = type.displayText( session, name );
								m_errors.add( LocalizedMessage.getSimpleMessage( "USERVALIDATOR_USER_NAME" ) + " " + name );
							}
						}
					}
				}
			}
			catch ( Throwable th ) {
				throw new RuntimeException( Helper.throwable2String( th ) );
			}
		}


		// Not filled Location records give a warning
		RecordIterator userLoc = getIterator( m_userLocationTable );
		boolean isValid = false;
		Set<String> locs = new HashSet<String>();
		if ( !userLoc.isEmpty( session ) ) {
			while ( userLoc.next( session ) ) {
				String locID = userLoc.getValue( session, "ULLI" );
				if ( locID == null || "".equals( locID ) ) {
					m_warnings.add( LocalizedMessage.getSimpleMessage( "USERVALIDATOR_LOCATION_SHOULD_BE_SELECTED" ) );
				}
				else {
					for ( RecordIterator loc = userLoc.getIterator( m_locationTable ); loc.next( session ); ) {
						String del = loc.getValue( session, "LODD" );
						String name = loc.getValue( session, "LONA" );
						if ( "yes".equals( del ) ) {
							m_warnings.add( LocalizedMessage.getSimpleMessage( "USERVALIDATOR_DELETED_LOCATION" ) + ( name == null ? "" : " " + name ) );
						}
						else {
							isValid = true;
						}
						if ( name != null ) {
							if ( locs.contains( name ) ) {
								m_warnings.add( LocalizedMessage.getSimpleMessage( "USERVALIDATOR_MUTIPLE_LOCATION" ) + name );
							}
							locs.add( name );
						}
					}
				}
			}
		}

		// if we have User Manage but no Admin right, then we want some managed groups
		boolean hasGroup = false;
		boolean emptyGroup = false;
		RecordIterator userGroup = getIterator( m_userGroupTable );
		if ( !userGroup.isEmpty( session ) ) {
			while ( userGroup.next( session ) ) {
				String grid = userGroup.getValue( session, "UGGI" );
				if ( grid != null && !"".equals( grid ) ) {
					hasGroup = true;
				}
				else {
					emptyGroup = true;
				}
			}
		}
		String rights = getValue( session, "URGT" );
		// TODO: temporary removed until UserManage right is properly implemented
		//		if ( IMTypeur.rightsIncluded( rights, PacsRight.UserManage.letterString() ) &&
		//				!IMTypeur.rightsIncluded( rights, PacsRight.Admin.letterString() + PacsRight.Support.letterString() + PacsRight.SupportRightForSupportUser.letterString() ) &&
		//				!hasGroup ) {
		//			m_warnings.add( LocalizedMessage.getSimpleMessage( "USERVALIDATOR_GROUP_MUST_BE_ADDED" ) );
		//		}
		//		else 
		if ( emptyGroup ) {
			m_warnings.add( LocalizedMessage.getSimpleMessage( "USERVALIDATOR_GROUP_EMPTY" ) );
		}

		if ( ( //!IMTypeur.rightsIncluded( rights, PacsRight.UserManage.letterString() ) ||		// TODO: temporary removed until UserManage right is properly implemented
		IMTypeur.rightsIncluded( rights, PacsRight.Admin.letterString() + PacsRight.Support.letterString() + PacsRight.SupportRightForSupportUser.letterString() ) ) &&
				hasGroup ) {
			m_warnings.add( LocalizedMessage.getSimpleMessage( "USERVALIDATOR_GROUP_SHOULD_BE_REMOVED" ) );
		}

		//Saving a User with 'Edit Report' right and without a Location should be forbidden
		// Removed from the default 
		//			if ( !isValid && IMTypeur.rightsIncluded( rights, IMTypeur.REPORT_RIGHT ) ) {
		//				m_errors.add( LocalizedMessage.getSimpleMessage( "USERVALIDATOR_NOLOCATION" ) );
		//			}

		String target = getValue( session, "USAT" );
		if ( target == null ) {
			target = IMAdminToolbox.AUTH_GRP;
		}
		if ( IMAdminToolbox.AUTH_GRP.equals( target ) ) {
			String group = getValue( session, "UGRP" );
			try {
				target = IMAdminToolbox.getGroupAuthTarget( group );
			}
			catch ( IMException | SQLException e ) {
				// group missing / should not happen
				throw new RuntimeException( "Getting auth target for group " + group + " failed: " + Helper.toString( e ) );
			}
		}
		if ( IMAdminToolbox.AUTH_SYS.equals( target ) ) {
			target = IMAdminToolbox.getSysAuthTarget();
		}
		if ( IMAdminToolbox.AUTH_EP.equals( target ) ) {
			//handlePw( false ); // this may not be false, I just guesed during merging
		}

	}

}
